#!/usr/bin/env python3

"""
Copyright (c) 2025 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.

This file incorporates work covered by the following copyright and
permission notice:

    Copyright (c) 2019 Максим-Стукало <https://stackoverflow.com/users/6709778/Максим-Стукало>

    CC BY-SA 4.0
"""

import collections
import json
import os
import platform
import signal
import subprocess
import sys
import time
import webbrowser
from datetime import datetime, timedelta
from pathlib import Path

import click
import requests

profile_path: Path


# https://stackoverflow.com/a/58323807
class OrderedGroup(click.Group):
    def __init__(self, name=None, commands=None, **attrs):
        super(OrderedGroup, self).__init__(name, commands, **attrs)
        self.commands = commands or collections.OrderedDict()

    def list_commands(self, ctx):
        return self.commands


@click.group(cls=OrderedGroup)
@click.option(
    "--profile",
    "-p",
    required=True,
    type=click.Path(
        file_okay=False,
        writable=True,
        path_type=Path,
    ),
)
def cli(profile):
    global profile_path
    profile_path = profile


@cli.command()
def status():
    "获取mower-ng实例状态"
    import psutil

    from mower.utils.network import is_port_in_use

    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return

    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    pid = data["pid"]
    port = data["port"]
    if not psutil.pid_exists(pid):
        click.echo("mower-ng实例未启动，上次异常退出")
        return

    if not is_port_in_use(port):
        click.echo("mower-ng正在启动……")
        return

    url = f"http://127.0.0.1:{port}"
    r = requests.get(f"{url}/scheduler", timeout=5)
    data = r.json()
    idle = data["idle"]
    next_time = data["next_time"]
    if not idle:
        click.echo("正在执行任务")
        return
    if next_time is None:
        click.echo("mower-ng进程已启动，调度器未运行")
        return
    next_time = datetime.fromisoformat(next_time)
    click.echo(f"调度器正在休息，下次任务在{next_time.strftime('%H:%M:%S')}")


@cli.command()
@click.option("--timeout", "-t", type=float, default=10, help="启动超时时间（秒）")
def launch(timeout):
    "启动mower-ng进程"
    from mower.utils.network import is_port_in_use

    creation_flags = 0
    if platform.system().lower() == "windows":
        creation_flags = subprocess.CREATE_NO_WINDOW

    folder = os.path.dirname(__file__)
    script_path = os.path.join(folder, "webui.py")
    subprocess.Popen(
        [sys.executable, script_path, str(profile_path)],
        cwd=folder,
        creationflags=creation_flags,
        stdin=subprocess.DEVNULL,
        stdout=subprocess.DEVNULL,
        stderr=subprocess.DEVNULL,
    )

    lock_file = profile_path / "lockfile.json"
    start_time = datetime.now()
    while True:
        time.sleep(0.5)
        if not lock_file.exists():
            continue
        with lock_file.open("r", encoding="utf-8") as f:
            data = json.load(f)
        port = data["port"]
        if is_port_in_use(port):
            click.echo("mower-ng已启动")
            break
        if datetime.now() - start_time > timedelta(seconds=timeout):
            click.echo("启动超时！")
            break
        click.echo("等待mower-ng启动……")


@cli.command()
def exit():
    "停止mower-ng进程"
    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    port = data["port"]
    token = data["token"]
    url = f"http://127.0.0.1:{port}"
    requests.get(f"{url}/exit", headers={"token": token}, timeout=5)


@cli.command()
@click.option(
    "--signal",
    "signal_name",
    type=click.Choice(["SIGINT", "SIGTERM", "SIGKILL"]),
    default=str("SIGINT"),
    help="信号量",
)
def kill(signal_name):
    "强制退出mower-ng进程"
    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    pid = data["pid"]
    os.kill(pid, getattr(signal, signal_name))


@cli.command()
def start():
    "开始运行调度器"
    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    port = data["port"]
    token = data["token"]
    url = f"http://127.0.0.1:{port}"
    requests.get(f"{url}/start", headers={"token": token}, timeout=5)
    click.echo("开始运行调度器")


@cli.command()
def stop():
    "停止运行调度器"
    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    port = data["port"]
    token = data["token"]
    url = f"http://127.0.0.1:{port}"
    r = requests.get(f"{url}/stop", headers={"token": token}, timeout=15)
    if r.text == "true":
        click.echo("成功停止调度器")
    else:
        click.echo("调度器仍在运行")


@cli.command()
@click.option("--title", help="设置网页标题")
def webui(title):
    "在浏览器中打开网页面板"
    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    port = data["port"]
    token = data["token"]

    url = f"http://127.0.0.1:{port}"

    if title is not None:
        requests.post(
            f"{url}/profile-name",
            headers={"token": token},
            json={"profile_name": title},
            timeout=5,
        )

    if token:
        url += f"?token={token}"
    click.echo(url)
    webbrowser.open(url)


@cli.command()
@click.option("--tail", "-n", type=int, help="显示日志的行数")
def log(tail):
    "通过WebSocket获取日志"
    from simple_websocket import Client

    lock_file = profile_path / "lockfile.json"
    if not lock_file.exists():
        click.echo("mower-ng实例未启动")
        return
    with lock_file.open("r", encoding="utf-8") as f:
        data = json.load(f)
    port = data["port"]

    ws = Client.connect(f"ws://127.0.0.1:{port}/ws")
    data = json.loads(ws.receive())
    if data["type"] == "log":
        log_lines = data["data"].split("\n")
        if tail is not None:
            log_lines = log_lines[-tail:]
        click.echo("\n".join(log_lines))
    ws.close()


if __name__ == "__main__":
    cli(auto_envvar_prefix="MOWER_NG")
