import json
import os
from datetime import datetime
from multiprocessing import Pool

import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont, ImageOps
from skimage.metrics import structural_similarity

now = datetime.now()
# 配置路径
REF_DIR = r"depot_test\output/test/origin"
ROI_DIR = r"depot_test\output/test/result"
IMG_PATH = r"depot_test\result_refined.png"
REF_TABLE_JSON = "./ArknightsGameData/zh_CN/gamedata/excel/item_table.json"
ICON_DIR = "./ArknightsResource/items/"

# SSIM阈值
SSIM_THRESHOLD = 0.01

# 圆检测参数
HOUGH_PARAMS = dict(
    dp=5, minDist=230, param1=50, param2=30, minRadius=90, maxRadius=100
)
CROP_SIZE = 130
BORDER = 26
SECONDARY_SLICE = (slice(30, 140), slice(50, 160))


def load_references(table_json, icon_dir):
    data = json.load(open(table_json, encoding="utf-8"))
    refs = {}
    size = CROP_SIZE * 2 - BORDER * 2
    for item in data.get("items", {}).values():
        t = item.get("classifyType")
        if t not in {"NORMAL", "CONSUME", "MATERIAL"}:
            continue
        path = os.path.join(icon_dir, f"{item['iconId']}.png")
        if not os.path.exists(path):
            continue
        im = Image.open(path).resize((size, size)).crop((50, 30, 160, 140)).convert("L")
        refs[item["name"]] = np.array(im)
    print(f"已加载 {len(refs)} 个参考图，保存于 {REF_DIR}")
    return refs


def process_circle(idx, circle, img, rois, size, dr):
    x, y, r = circle
    crop = img[
        max(0, y - CROP_SIZE) : min(img.shape[0], y + CROP_SIZE),
        max(0, x - CROP_SIZE) : min(img.shape[1], x + CROP_SIZE),
    ]
    c = crop[BORDER:-BORDER, BORDER:-BORDER]
    sec = c[SECONDARY_SLICE[0], SECONDARY_SLICE[1]]
    gray_sec = cv2.cvtColor(sec, cv2.COLOR_BGR2GRAY)

    # 保存 ROI 图像
    os.makedirs(ROI_DIR, exist_ok=True)
    roi_path = os.path.join(ROI_DIR, f"roi_{idx}.png")
    cv2.imwrite(roi_path, gray_sec)
    rois.append(gray_sec)
    return idx, gray_sec, roi_path


def detect_and_crop(image_path):
    img = cv2.imread(image_path)
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    circles = cv2.HoughCircles(gray, cv2.HOUGH_GRADIENT, **HOUGH_PARAMS)
    if circles is None:
        print("未检测到圆形区域")
        return []
    circles = np.round(circles[0]).astype(int)
    rois = []
    size = CROP_SIZE * 2 - BORDER * 2
    dr = img.max() - img.min()

    # 使用多进程加速
    with Pool() as pool:
        results = pool.starmap(
            process_circle,
            [(idx, circle, img, rois, size, dr) for idx, circle in enumerate(circles)],
        )

    return [roi for idx, roi, path in results]


def process_match(idx, roi, refs, thresh):
    best, score = "Unknown", -1
    dr = roi.max() - roi.min()
    for name, ref in refs.items():
        if roi.shape != ref.shape:
            continue
        s = structural_similarity(roi, ref, data_range=dr)
        if s > score:
            best, score = name, s
    if score >= thresh:
        return idx, best, score
    return idx, None, score


def match_ssim(rois, refs, thresh=SSIM_THRESHOLD):
    from multiprocessing import Pool

    args = [(idx, roi, refs, thresh) for idx, roi in enumerate(rois)]
    with Pool(processes=5) as pool:
        results = pool.starmap(process_match, args)

    stats = {}
    match_idx = {}
    for idx, name, score in results:
        if name:
            stats[name] = stats.get(name, 0) + 1
            match_idx[idx] = name
        print(f"ROI {idx} 匹配结果: {name if name else 'Unknown'} (SSIM={score:.3f})")
    return stats, match_idx


def display_matches(rois, match_idx, refs):
    ROW_LIMIT = 10  # 每行最多10个
    blank_ref = next(iter(refs.values()))  # 取一个参考图的尺寸
    blank_img = Image.new(
        "RGB", (blank_ref.shape[1], blank_ref.shape[0]), (200, 200, 200)
    )  # 灰色占位图

    combined_images = []
    row_images = []

    row_width = 0
    max_height = 0

    for idx in range(len(rois)):
        roi_img = Image.fromarray(rois[idx]).convert("RGB")
        ref_name = match_idx.get(idx)
        if ref_name:
            ref_img = Image.fromarray(refs[ref_name]).convert("RGB")
        else:
            ref_img = blank_img.copy()

        combined_width = roi_img.width + ref_img.width
        combined_height = max(roi_img.height, ref_img.height)

        combined = Image.new("RGB", (combined_width, combined_height), (255, 255, 255))
        combined.paste(roi_img, (0, 0))
        combined.paste(ref_img, (roi_img.width, 0))

        draw = ImageDraw.Draw(combined)

        font = ImageFont.truetype("msyh.ttc", 20)

        label = f"ROI {idx}: {ref_name if ref_name else 'Unknown'}"
        draw.text((5, 5), label, fill=(255, 0, 0), font=font)
        combined = ImageOps.expand(combined, border=2, fill=(0, 0, 0))
        row_images.append(combined)
        row_width += combined_width
        max_height = max(max_height, combined_height)

        if len(row_images) == ROW_LIMIT or idx == len(rois) - 1:
            row_img = Image.new("RGB", (row_width, max_height), (255, 255, 255))
            x_offset = 0
            for img in row_images:
                row_img.paste(img, (x_offset, 0))
                x_offset += img.width
            combined_images.append(row_img)
            row_images = []
            row_width = 0
            max_height = 0

    total_width = max(img.width for img in combined_images)
    total_height = sum(img.height for img in combined_images)

    final_img = Image.new("RGB", (total_width, total_height), (255, 255, 255))
    y_offset = 0
    for img in combined_images:
        final_img.paste(img, (0, y_offset))
        y_offset += img.height

    final_img.save("all_matches.png")  # 或 final_img.save("all_matches.png")


if __name__ == "__main__":
    refs = load_references(REF_TABLE_JSON, ICON_DIR)
    rois = detect_and_crop(IMG_PATH)
    res, match_idx = match_ssim(rois, refs)

    print("最终识别结果:", res)
    display_matches(rois, match_idx, refs)
    print(datetime.now() - now)
