import json
import lzma
import os
import pickle
from itertools import dropwhile, islice

import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont, ImageOps
from skimage.feature import hog
from sklearn.neighbors import KNeighborsClassifier


def crop_dict_after_key(refs, target_key, max_items=500, return_keys=False):
    """
    裁切字典，返回 target_key 及其之后的项（默认返回键列表）

    参数:
        refs: 输入字典
        target_key: 起始键（包含该键）
        max_items: 最多返回的项数
        return_keys: 是否返回键列表（False则返回字典）
    """
    if target_key is None:
        items = list(islice(refs.items(), max_items))
    else:
        # 获取 target_key 之后的所有项
        cropped = dict(dropwhile(lambda item: item[0] != target_key, refs.items()))
        items = list(islice(cropped.items(), max_items))

    if return_keys:
        if target_key is None:
            return None
        return [key for key, _ in items]  # 返回键列表
    else:
        return dict(items)  # 返回字典（原逻辑）


def 提取特征点(模板):
    """提取HOG特征（处理彩色图像）"""
    hog_features = hog(
        模板,
        orientations=18,
        pixels_per_cell=(8, 8),
        cells_per_block=(2, 2),
        block_norm="L2-Hys",
        transform_sqrt=True,
        channel_axis=None,
    )
    return hog_features


class DepotMatcher:
    def __init__(
        self,
        ref_table_json="./ArknightsGameData/zh_CN/gamedata/excel/item_table.json",
        icon_dir="./ArknightsResource/items/",
        ref_dir="depot_test/output/test/origin",
        roi_dir="depot_test/output/test/result",
        img_path="depot_test/output/result_template.png",
        hough_params=None,
        crop_size=130,
        border=26,
        secondary_slice=(slice(30, 140), slice(50, 160)),
    ):
        # 初始化路径配置
        self.REF_DIR = ref_dir
        self.ROI_DIR = roi_dir
        self.IMG_PATH = img_path
        self.REF_TABLE_JSON = ref_table_json
        self.ICON_DIR = icon_dir

        # 初始化算法参数
        self.HOUGH_PARAMS = hough_params or dict(
            dp=5, minDist=230, param1=50, param2=30, minRadius=90, maxRadius=100
        )
        self.CROP_SIZE = crop_size
        self.BORDER = border
        self.SECONDARY_SLICE = secondary_slice

        # 运行时数据存储
        self.refs = None
        self.rois = []
        self.template_results = []
        self.knn_results = []

    def load_references(self):
        """加载物品图标参考图（保留彩色）"""
        data = json.load(open(self.REF_TABLE_JSON, encoding="utf-8"))
        self.refs = {}
        size = self.CROP_SIZE * 2 - self.BORDER * 2

        # 首先收集所有带有sortId的物品
        items_with_sort = []
        for item in data.get("items", {}).values():
            if item.get("classifyType") not in {"NORMAL", "CONSUME", "MATERIAL"}:
                continue

            path = os.path.join(self.ICON_DIR, f"{item['iconId']}.png")
            if not os.path.exists(path):
                continue

            # 保留彩色图像
            im = Image.open(path).resize((size, size)).crop((50, 30, 160, 140))
            items_with_sort.append(
                {
                    "name": item["name"],
                    "array": np.array(im),
                    "sortId": item.get("sortId", 0),
                }
            )

        # 按sortId排序
        items_with_sort.sort(key=lambda x: x["sortId"])

        # 创建最终的refs字典
        for item in items_with_sort:
            self.refs[item["name"]] = item["array"]

        print(f"已加载 {len(self.refs)} 个参考图 (按sortId排序)")
        return self

    def _process_circle(self, idx, circle, img):
        """处理单个圆形区域，返回彩色图像数据"""
        x, y, r = circle
        # 裁剪包含圆形的更大区域
        crop = img[
            max(0, y - self.CROP_SIZE) : min(img.shape[0], y + self.CROP_SIZE),
            max(0, x - self.CROP_SIZE) : min(img.shape[1], x + self.CROP_SIZE),
        ]

        # 提取核心的彩色ROI区域
        color_roi = crop[self.BORDER : -self.BORDER, self.BORDER : -self.BORDER]

        # 保存彩色ROI文件
        os.makedirs(self.ROI_DIR, exist_ok=True)
        color_roi_path = os.path.join(self.ROI_DIR, f"color_roi_{idx}.png")
        cv2.imwrite(color_roi_path, color_roi)

        # 提取用于匹配的彩色区域
        color_sec = color_roi[self.SECONDARY_SLICE[0], self.SECONDARY_SLICE[1]]

        return idx, color_sec, color_roi

    def detect_and_crop(self):
        """检测并裁剪圆形区域"""
        img = cv2.imread(self.IMG_PATH)
        gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
        circles = cv2.HoughCircles(gray, cv2.HOUGH_GRADIENT, **self.HOUGH_PARAMS)

        if circles is None:
            print("未检测到圆形区域")
            return self

        # 处理检测到的圆形
        circles = np.round(circles[0]).astype(int)
        circles = sorted(circles, key=lambda c: (c[0], c[1]))  # 按坐标排序

        self.rois = []
        for idx, circle in enumerate(circles):
            result = self._process_circle(idx, circle, img)
            self.rois.append(result)

        return self

    def 训练并保存knn模型(self, images, labels, filename):
        """训练并保存KNN模型"""
        knn_classifier = KNeighborsClassifier(
            weights="distance", n_neighbors=1, n_jobs=1
        )
        knn_classifier.fit(images, labels)

        with lzma.open(filename, "wb") as f:
            pickle.dump(knn_classifier, f)

        return knn_classifier

    def 训练knn模型(self, 模型保存路径="depot_knn_model.xz"):
        """训练并保存KNN模型（使用彩色图像）"""
        if not self.refs:
            print("错误：请先加载参考图像！")
            return None

        # 准备训练数据
        images = []
        labels = []

        for name, img_array in self.refs.items():
            features = 提取特征点(cv2.cvtColor(img_array, cv2.COLOR_BGR2GRAY))
            images.append(features)
            labels.append(name)

        # 训练模型
        self.knn_model = self.训练并保存knn模型(images, labels, 模型保存路径)
        print(f"KNN模型训练完成，已保存到: {模型保存路径}")
        return self

    def 使用knn预测(self, 测试图像, allowed_names=None):
        """使用KNN模型进行预测"""

        features = 提取特征点(测试图像)

        # 预测
        try:
            if allowed_names is not None:
                # 动态过滤训练数据
                label_names = self.knn_model.classes_
                allowed_indices = [
                    i for i, name in enumerate(label_names) if name in allowed_names
                ]
                mask = [label in allowed_indices for label in self.knn_model._y]

                self.knn_model._fit_X = self.knn_model._fit_X[mask]
                self.knn_model._y = np.array(self.knn_model._y)[mask]
            预测结果 = self.knn_model.predict([features])

            return 预测结果[0]
        except Exception as e:
            print(f"预测失败: {str(e)}")
            return None, None

    def match_items(
        self,
        knn_model_path="depot_knn_model.xz",
        template_threshold=0.2,
        SIMM_threshold=0.2,
    ):
        """合并KNN和模板匹配方法"""
        self.knn_results = []
        self.template_results = []
        newstart = None  # 初始化newstart变量
        with lzma.open(knn_model_path, "rb") as f:
            self.knn_model = pickle.load(f)
        for idx, color_sec_np, _ in self.rois:
            # KNN预测
            roi_gray = cv2.cvtColor(color_sec_np, cv2.COLOR_RGB2GRAY)
            allowed_names = crop_dict_after_key(self.refs, newstart, return_keys=True)
            knn_name = self.使用knn预测(roi_gray, allowed_names=allowed_names)

            self.knn_results.append((idx, knn_name))

            # 模板匹配
            best_template, max_val = "Unknown", -1.0

            for name, ref in crop_dict_after_key(self.refs, newstart).items():
                ref_gray = cv2.cvtColor(ref, cv2.COLOR_RGB2GRAY)
                res = cv2.matchTemplate(roi_gray, ref_gray, cv2.TM_CCOEFF_NORMED)
                val = res.max()

                if val > max_val:
                    best_template, max_val = name, val

            if max_val >= template_threshold:
                self.template_results.append((idx, best_template, max_val))
            else:
                self.template_results.append((idx, None, max_val))

            # 更新newstart逻辑
            if knn_name == best_template:
                newstart = knn_name
            else:
                newstart = newstart  # 保持不变

            print(
                f"ROI {idx}: KNN={knn_name}, Template={best_template},Score={max_val:.2f}, newstart={newstart}"
            )

        return self

    def display_results(self):
        """可视化匹配结果"""
        ROW_LIMIT = 9

        # 获取一个参考图像的尺寸作为空白图像的基础
        blank_ref_np = next(iter(self.refs.values()))
        blank_img_pil = Image.new(
            "RGB", (blank_ref_np.shape[1], blank_ref_np.shape[0]), (200, 200, 200)
        )

        combined_images = []
        current_row_images = []
        current_row_width = 0
        max_row_height = 0

        for idx, color_sec_np, color_roi_data in self.rois:
            color_roi_data = Image.fromarray(
                cv2.cvtColor(color_roi_data, cv2.COLOR_BGR2RGB)
            )
            color_sec_np = Image.fromarray(
                cv2.cvtColor(color_sec_np, cv2.COLOR_BGR2RGB)
            )

            # 获取模板匹配结果
            t_res_name = next(
                (name for i, name, val in self.template_results if i == idx), None
            )
            t_res_val = next(
                (val for i, name, val in self.template_results if i == idx), 0
            )
            t_ref_img = (
                Image.fromarray(self.refs[t_res_name]).convert("RGB")
                if t_res_name and t_res_name in self.refs
                else blank_img_pil.copy()
            )

            # 获取KNN匹配结果
            k_res_details = next(
                (d for d in getattr(self, "knn_results", []) if d[0] == idx), None
            )
            k_res_name = k_res_details[1] if k_res_details else None
            k_ref_img = (
                Image.fromarray(self.refs[k_res_name]).convert("RGB")
                if k_res_name and k_res_name in self.refs
                else blank_img_pil.copy()
            )

            # 计算组合尺寸
            combined_width = (
                color_roi_data.width
                + color_sec_np.width
                + t_ref_img.width
                + k_ref_img.width
            )
            combined_height = max(
                color_roi_data.height,
                color_sec_np.height,
                t_ref_img.height,
                k_ref_img.height,
            )

            # 创建组合图像
            combined = Image.new(
                "RGB", (combined_width, combined_height), (255, 255, 255)
            )
            x_offset = 0

            # 粘贴各个部分
            combined.paste(color_roi_data, (x_offset, 0))
            x_offset += color_roi_data.width

            combined.paste(color_sec_np, (x_offset, 0))
            x_offset += color_sec_np.width

            combined.paste(t_ref_img, (x_offset, 0))
            x_offset += t_ref_img.width

            combined.paste(k_ref_img, (x_offset, 0))
            x_offset += k_ref_img.width

            # 添加标注
            draw = ImageDraw.Draw(combined)
            font = ImageFont.truetype("msyh.ttc", 16)

            label = (
                f"ROI {idx}\n"
                f"模版匹配: {t_res_name or 'None'} ({t_res_val:.2f})\n"
                f"Hog+Knn: {k_res_name or 'None'}"
            )

            text_color = (255, 0, 0) if t_res_name == k_res_name else (0, 0, 0)

            draw.text(
                (color_roi_data.width, color_sec_np.height),
                label,
                fill=text_color,
                font=font,
            )

            # 添加边框
            combined_bordered = ImageOps.expand(combined, border=2, fill=(0, 0, 0))
            current_row_images.append(combined_bordered)
            current_row_width += combined_bordered.width
            max_row_height = max(max_row_height, combined_bordered.height)

            # 检查是否需要换行
            if len(current_row_images) == ROW_LIMIT:
                row_img = Image.new(
                    "RGB", (current_row_width, max_row_height), (255, 255, 255)
                )
                x = 0
                for img in current_row_images:
                    row_img.paste(img, (x, 0))
                    x += img.width
                combined_images.append(row_img)
                current_row_images = []
                current_row_width = 0
                max_row_height = 0

        # 处理最后一行
        if current_row_images:
            row_img = Image.new(
                "RGB", (current_row_width, max_row_height), (255, 255, 255)
            )
            x = 0
            for img in current_row_images:
                row_img.paste(img, (x, 0))
                x += img.width
            combined_images.append(row_img)

        # 生成最终图像
        if combined_images:
            total_height = sum(img.height for img in combined_images)
            max_width = max(img.width for img in combined_images)
            final_img = Image.new("RGB", (max_width, total_height), (255, 255, 255))

            y = 0
            for img in combined_images:
                final_img.paste(img, (0, y))
                y += img.height

            output_path = "depot_test/output/matches_all.png"
            os.makedirs(os.path.dirname(output_path), exist_ok=True)
            final_img.save(output_path)
            print(f"结果图像已保存至: {output_path}")

        return self


if __name__ == "__main__":
    # 使用示例
    matcher = DepotMatcher()
    matcher.load_references().训练knn模型()
    from datetime import datetime

    now_time = datetime.now()
    matcher.detect_and_crop().match_items().display_results()

    # 打印结果对比
    print("\n=== 结果对比 ===")
    for idx in range(len(matcher.rois)):
        t_res, t_val = next(
            ((name, val) for i, name, val in matcher.template_results if i == idx), None
        )

        k_res = next((name for i, name in matcher.knn_results if i == idx), None)

        print(f"ROI {idx}:")
        print(f"  Template匹配: {t_res} ({t_val:.2f})")
        print(f"  HOG+KNN匹配: {k_res} ")
    print(datetime.now() - now_time)
