import json
import os
from datetime import datetime

import cv2
import numpy as np
from PIL import Image, ImageDraw, ImageFont, ImageOps
from skimage.metrics import structural_similarity as ssim

# 配置路径
REF_DIR = r"depot_test\output/test/origin"
ROI_DIR = r"depot_test\output/test/result"
IMG_PATH = r"depot_test\output\result_template.png"
REF_TABLE_JSON = "./ArknightsGameData/zh_CN/gamedata/excel/item_table.json"
ICON_DIR = "./ArknightsResource/items/"

# 参数
HOUGH_PARAMS = dict(
    dp=5, minDist=230, param1=50, param2=30, minRadius=90, maxRadius=100
)
CROP_SIZE = 130
BORDER = 26
SECONDARY_SLICE = (slice(30, 140), slice(50, 160))


def load_references(table_json, icon_dir):
    data = json.load(open(table_json, encoding="utf-8"))
    refs = {}
    size = CROP_SIZE * 2 - BORDER * 2
    for item in data.get("items", {}).values():
        if item.get("classifyType") not in {"NORMAL", "CONSUME", "MATERIAL"}:
            continue
        path = os.path.join(icon_dir, f"{item['iconId']}.png")
        if not os.path.exists(path):
            continue
        im = Image.open(path).resize((size, size)).crop((50, 30, 160, 140)).convert("L")
        refs[item["name"]] = np.array(im)
    print(f"已加载 {len(refs)} 个参考图")
    return refs


def process_circle(idx, circle, img):
    x, y, r = circle
    crop = img[
        max(0, y - CROP_SIZE) : min(img.shape[0], y + CROP_SIZE),
        max(0, x - CROP_SIZE) : min(img.shape[1], x + CROP_SIZE),
    ]
    # Save color ROI
    os.makedirs(ROI_DIR, exist_ok=True)
    color_roi_path = os.path.join(ROI_DIR, f"color_roi_{idx}.png")
    cv2.imwrite(color_roi_path, crop[BORDER:-BORDER, BORDER:-BORDER])

    c = crop[BORDER:-BORDER, BORDER:-BORDER]
    sec = c[SECONDARY_SLICE[0], SECONDARY_SLICE[1]]
    gray_sec = cv2.cvtColor(sec, cv2.COLOR_BGR2GRAY)
    roi_path = os.path.join(ROI_DIR, f"roi_{idx}.png")
    cv2.imwrite(roi_path, gray_sec)
    return idx, gray_sec, roi_path, color_roi_path


def detect_and_crop(image_path):
    img = cv2.imread(image_path)
    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    circles = cv2.HoughCircles(gray, cv2.HOUGH_GRADIENT, **HOUGH_PARAMS)
    if circles is None:
        print("未检测到圆形区域")
        return []

    # Convert and sort circles by y then x coordinate
    circles = np.round(circles[0]).astype(int)
    circles = sorted(circles, key=lambda c: (c[0], c[1]))  # Sort by y then x

    results = []
    for idx, circle in enumerate(circles):
        result = process_circle(idx, circle, img)
        results.append(result)

    return results


def match_template(rois, refs, thresh):
    results = []
    for idx, roi, _, _ in rois:
        best, max_val = "Unknown", -1.0
        roi_f = roi.astype(np.float32) / 255.0
        for name, ref in refs.items():
            ref_f = ref.astype(np.float32) / 255.0
            if roi_f.shape != ref_f.shape:
                continue
            res = cv2.matchTemplate(roi_f, ref_f, cv2.TM_CCOEFF_NORMED)
            val = float(res.max())
            if val > max_val:
                best, max_val = name, val
        if max_val >= thresh:
            results.append((idx, best, max_val))
        else:
            results.append((idx, None, max_val))

    return results


def match_ssim(rois, refs, thresh):
    results = []

    for idx, roi, _, _ in rois:
        best_match = "Unknown"
        max_combined_score = -1.0
        best_ssim = 0
        best_hist = 0
        best_edge = 0

        # 预处理ROI
        roi_gray = roi if len(roi.shape) == 2 else cv2.cvtColor(roi, cv2.COLOR_BGR2GRAY)
        roi_edges = cv2.Canny(roi_gray, 50, 150)
        roi_hist = cv2.calcHist([roi_gray], [0], None, [256], [0, 256])
        cv2.normalize(roi_hist, roi_hist, alpha=0, beta=1, norm_type=cv2.NORM_MINMAX)

        for name, ref in refs.items():
            if roi_gray.shape != ref.shape:
                continue

            # 1. 计算SSIM相似度
            ssim_score, _ = ssim(roi_gray, ref, full=True)

            # 2. 计算直方图相似度
            ref_hist = cv2.calcHist([ref], [0], None, [256], [0, 256])
            cv2.normalize(
                ref_hist, ref_hist, alpha=0, beta=1, norm_type=cv2.NORM_MINMAX
            )
            hist_score = cv2.compareHist(roi_hist, ref_hist, cv2.HISTCMP_CORREL)

            # 3. 计算边缘相似度
            ref_edges = cv2.Canny(ref, 50, 150)
            edge_intersection = np.sum(roi_edges * ref_edges)
            edge_union = np.sum(roi_edges + ref_edges)
            edge_score = edge_intersection / edge_union if edge_union > 0 else 0

            # 加权综合评分 (可调整权重)
            combined_score = 0.6 * ssim_score + 0.2 * hist_score + 0.2 * edge_score

            if combined_score > max_combined_score:
                best_match = name
                max_combined_score = combined_score
                best_ssim = ssim_score
                best_hist = hist_score
                best_edge = edge_score

        # 动态阈值调整 (基于图像复杂度)
        roi_complexity = np.std(roi_gray) / 255.0
        dynamic_thresh = thresh * (1 + 0.3 * roi_complexity)

        if max_combined_score >= dynamic_thresh:
            results.append(
                (idx, best_match, max_combined_score, best_ssim, best_hist, best_edge)
            )
        else:
            results.append(
                (idx, None, max_combined_score, best_ssim, best_hist, best_edge)
            )

    return results


def display_matches(rois, template_results, ssim_results, refs):
    ROW_LIMIT = 9
    blank_ref = next(iter(refs.values()))  # 取一个参考图的尺寸
    blank_img = Image.new(
        "RGB", (blank_ref.shape[1], blank_ref.shape[0]), (200, 200, 200)
    )  # 灰色占位图

    combined_images = []
    row_images = []

    row_width = 0
    max_height = 0

    for item in rois:
        idx, _, _, color_path = item
        # Load color ROI image
        roi_img = Image.open(color_path).convert("RGB")

        # Template matching result
        t_res = next((name for i, name, val in template_results if i == idx), None)
        t_val = next((val for i, name, val in template_results if i == idx), 0)
        if t_res is not None:
            t_ref_img = Image.fromarray(refs[t_res]).convert("RGB")
        else:
            t_ref_img = blank_img.copy()

        # SSIM result - we need to get the detailed scores
        s_res = next((name for i, name, val in ssim_results if i == idx), None)
        s_val = next((val for i, name, val in ssim_results if i == idx), 0)
        if s_res is not None:
            s_ref_img = Image.fromarray(refs[s_res]).convert("RGB")
        else:
            s_ref_img = blank_img.copy()

        # Combine Template Matching result (left) and SSIM result (right)
        combined_width = roi_img.width + t_ref_img.width + s_ref_img.width
        combined_height = max(roi_img.height, t_ref_img.height, s_ref_img.height)

        combined = Image.new("RGB", (combined_width, combined_height), (255, 255, 255))
        combined.paste(roi_img, (0, 0))
        combined.paste(t_ref_img, (roi_img.width, 0))
        combined.paste(s_ref_img, (roi_img.width + t_ref_img.width, 0))

        draw = ImageDraw.Draw(combined)
        font = ImageFont.truetype("msyh.ttc", 20)

        # Get the detailed scores from the SSIM matching results
        ssim_details = next(
            (details for i, details in enumerate(ssim_results) if i == idx),
            (idx, None, 0, 0, 0, 0),  # Default values if not found
        )
        best_ssim = ssim_details[3] if len(ssim_details) > 3 else 0
        best_hist = ssim_details[4] if len(ssim_details) > 4 else 0
        best_edge = ssim_details[5] if len(ssim_details) > 5 else 0

        label = (
            f"ROI {idx} {best_ssim:.3f}{best_hist:.3f} {best_edge:.3f}\n"
            f"T({t_res if t_res else 'None'}, {t_val:.3f})\n"
            f"S({s_res if s_res else 'None'}, {s_val:.3f})"
        )

        if t_res == s_res:
            draw.text(
                (roi_img.width, t_ref_img.height), label, fill=(255, 0, 0), font=font
            )
        else:
            draw.text(
                (roi_img.width, t_ref_img.height), label, fill=(255, 0, 255), font=font
            )
        combined = ImageOps.expand(combined, border=2, fill=(0, 0, 0))
        row_images.append(combined)
        row_width += combined_width
        max_height = max(max_height, combined_height)

        if len(row_images) == ROW_LIMIT or idx == len(rois) - 1:
            row_img = Image.new("RGB", (row_width, max_height), (255, 255, 255))
            x_offset = 0
            for img in row_images:
                row_img.paste(img, (x_offset, 0))
                x_offset += img.width
            combined_images.append(row_img)
            row_images = []
            row_width = 0
            max_height = 0

    total_width = max(img.width for img in combined_images)
    total_height = sum(img.height for img in combined_images)

    final_img = Image.new("RGB", (total_width, total_height), (255, 255, 255))
    y_offset = 0
    for img in combined_images:
        final_img.paste(img, (0, y_offset))
        y_offset += img.height

    final_img.save("depot_test/output/matches_all.png")


if __name__ == "__main__":
    refs = load_references(REF_TABLE_JSON, ICON_DIR)
    rois = detect_and_crop(IMG_PATH)

    now = datetime.now()

    Template_MATCH_THRESHOLD = 0.2
    print("\n=== 使用 Template Matching ===")
    results_template = match_template(rois, refs, Template_MATCH_THRESHOLD)
    print("\n模版匹配耗时:", datetime.now() - now)
    SSIM_MATCH_THRESHOLD = 0.05

    now = datetime.now()
    print("\n=== 使用 SSIM ===")
    results_ssim = match_ssim(rois, refs, SSIM_MATCH_THRESHOLD)
    print("\nSSIM匹配耗时:", datetime.now() - now)

    print("\n=== 结果对比 ===")
    for idx, _, _, _ in rois:
        # Template matching results
        t_res = next((name for i, name, val in results_template if i == idx), None)
        t_val = next((val for i, name, val in results_template if i == idx), 0)

        # SSIM results - now includes detailed metrics
        s_res = next(
            (name for i, name, val, ssim, hist, edge in results_ssim if i == idx), None
        )
        s_val = next(
            (val for i, name, val, ssim, hist, edge in results_ssim if i == idx), 0
        )
        s_ssim = next(
            (ssim for i, name, val, ssim, hist, edge in results_ssim if i == idx), 0
        )
        s_hist = next(
            (hist for i, name, val, ssim, hist, edge in results_ssim if i == idx), 0
        )
        s_edge = next(
            (edge for i, name, val, ssim, hist, edge in results_ssim if i == idx), 0
        )

        print(
            f"ROI {idx}:\n"
            f"  Template=({t_res if t_res else 'None'}, {t_val:.3f})\n"
            f"  SSIM=({s_res if s_res else 'None'}, {s_val:.3f})\n"
            f"  Details: SSIM={s_ssim:.3f}, Hist={s_hist:.3f}, Edge={s_edge:.3f}"
        )

    # Displaying matches side by side using the updated function
    display_matches(rois, results_template, results_ssim, refs)
