import glob
import os
from datetime import datetime

import cv2
import numpy as np


def load_images(input_dir):
    image_paths = sorted(glob.glob(os.path.join(input_dir, "*.png")))
    images = [
        cv2.imread(p)[140:1000, :] for p in image_paths if cv2.imread(p) is not None
    ]
    return image_paths, images


def save_image(output_path, image):
    cv2.imwrite(output_path, image)
    print(f"保存成功: {output_path}")


def match_template_stitch(image_paths, images, output_path):
    TEMPLATE_WIDTH = 100
    TEMPLATE_START_X = 400
    IMG_HEIGHT, IMG_WIDTH, _ = images[0].shape

    match_data = []
    image_cache = {p: img for p, img in zip(image_paths, images)}

    for i in range(len(image_paths) - 1):
        img_current = image_cache[image_paths[i]]
        img_next = image_cache[image_paths[i + 1]]

        template = img_next[:, TEMPLATE_START_X : TEMPLATE_START_X + TEMPLATE_WIDTH]

        result = cv2.matchTemplate(img_current, template, cv2.TM_CCOEFF_NORMED)
        _, max_val, _, max_loc = cv2.minMaxLoc(result)

        match_x = max_loc[0]
        split_x_current = match_x
        split_x_next = TEMPLATE_START_X

        match_data.append(
            {
                "img1_path": image_paths[i],
                "img2_path": image_paths[i + 1],
                "split_x_img1": split_x_current,
                "split_x_img2": split_x_next,
                "confidence": max_val,
            }
        )

    image_offsets = {image_paths[0]: 0}
    for match in match_data:
        img1 = match["img1_path"]
        img2 = match["img2_path"]
        offset_img1 = image_offsets[img1]
        offset_img2 = offset_img1 + match["split_x_img1"] - match["split_x_img2"]
        image_offsets[img2] = offset_img2

    last_image = image_paths[-1]
    total_width = image_offsets[last_image] + IMG_WIDTH
    stitched_img = np.zeros((IMG_HEIGHT, total_width, 3), dtype=np.uint8)

    for i, img_path in enumerate(image_paths):
        img = image_cache[img_path]
        x_start = image_offsets[img_path]
        copy_start_x = match_data[i - 1]["split_x_img2"] if i > 0 else 0
        source_part = img[:, copy_start_x:]
        dest_x = x_start + copy_start_x
        stitched_img[:, dest_x : dest_x + source_part.shape[1]] = source_part

    cv2.imwrite(output_path, stitched_img)
    print(f"模板匹配拼接完成，保存至: {output_path}")


def orb_stitch(image_paths, images, output_path):
    orb = cv2.ORB_create(nfeatures=1000)
    FLANN_INDEX_LSH = 6
    index_params = dict(
        algorithm=FLANN_INDEX_LSH, table_number=6, key_size=12, multi_probe_level=1
    )
    search_params = dict(checks=20)
    flann = cv2.FlannBasedMatcher(index_params, search_params)

    key_desc = [
        orb.detectAndCompute(cv2.cvtColor(img, cv2.COLOR_BGR2GRAY), None)
        for img in images
    ]

    offset_xs = []
    for i in range(len(images) - 1):
        kp1, des1 = key_desc[i]
        kp2, des2 = key_desc[i + 1]
        matches = flann.knnMatch(des1, des2, k=2)
        good = [
            m
            for m, n in matches
            if n.distance > 1e-6 and m.distance < 0.75 * n.distance
        ]

        if len(good) < 4:
            print(f"第{i}组匹配点过少：{len(good)}")
            offset_xs.append(0)
            continue

        src_pts = np.float32([kp1[m.queryIdx].pt for m in good]).reshape(-1, 1, 2)
        dst_pts = np.float32([kp2[m.trainIdx].pt for m in good]).reshape(-1, 1, 2)
        H, _ = cv2.findHomography(dst_pts, src_pts, cv2.RANSAC, 2.0)

        if H is None:
            print(f"第{i}组计算单应矩阵失败")
            offset_xs.append(0)
            continue

        offset_x = int(
            round(cv2.perspectiveTransform(np.float32([[[0, 0]]]), H)[0, 0, 0])
        )
        offset_xs.append(offset_x)

    absolute_xs = [0]
    for dx in offset_xs:
        absolute_xs.append(absolute_xs[-1] + dx)

    min_x = min(absolute_xs)
    max_x = max(absolute_xs[i] + img.shape[1] for i, img in enumerate(images))
    height = images[0].shape[0]
    width = max_x - min_x

    stitched = np.zeros((height, width, 3), dtype=np.uint8)

    for img, abs_x in zip(images, absolute_xs):
        x_start = abs_x - min_x
        x_end = x_start + img.shape[1]
        stitched[:, x_start:x_end] = img

    cv2.imwrite(output_path, stitched)
    print(f"ORB 拼接完成，保存至: {output_path}")


def stitcher_stitch(images, output_path):
    stitcher = cv2.Stitcher_create()
    status, pano = stitcher.stitch(images)
    if status == cv2.Stitcher_OK:
        save_image(output_path, pano)
    else:
        print(f"拼接失败，错误码：{status}")


def main(method="template"):
    input_dir = r"depot_test\Test"
    output_dir = r"depot_test\output"
    os.makedirs(output_dir, exist_ok=True)
    output_path = os.path.join(output_dir, f"result_{method}.png")

    start = datetime.now()
    image_paths, images = load_images(input_dir)

    if method == "template":
        match_template_stitch(image_paths, images, output_path)
    elif method == "orb":
        orb_stitch(image_paths, images, output_path)
    elif method == "stitcher":
        stitcher_stitch(images, output_path)
    else:
        print(f"未知方法: {method}")

    print("总耗时：", datetime.now() - start)


if __name__ == "__main__":
    main(method="template")
    main(method="orb")
    main(method="stitcher")
    output_dir = r"depot_test\output"
    img1 = cv2.imread(os.path.join(output_dir, "result_template.png"))
    img2 = cv2.imread(os.path.join(output_dir, "result_orb.png"))
    img3 = cv2.imread(os.path.join(output_dir, "result_stitcher.png"))

    # 确保宽度一致
    min_width = min(img1.shape[1], img2.shape[1], img3.shape[1])
    img1 = img1[:, :min_width]
    img2 = img2[:, :min_width]
    img3 = img3[:, :min_width]

    final_image = cv2.vconcat([img1, img2, img3])
    cv2.imwrite(os.path.join(output_dir, "result_all.png"), final_image)
    print("上下拼接完成，保存至:", os.path.join(output_dir, "result_all.png"))
