import glob
import os
import traceback

import cv2
import matplotlib.pyplot as plt
import numpy as np

# --- 全局设置 ---
plt.rcParams["font.sans-serif"] = ["Microsoft YaHei", "SimHei"] + plt.rcParams[
    "font.sans-serif"
]
plt.rcParams["axes.unicode_minus"] = False
print("已设置 Matplotlib 中文字体。")


# --- 核心功能函数 ---


def crop_images(img_a_full, img_b_full, crop_coordinates):
    """
    根据 crop_coordinates 裁剪图像 A 和 B。
    裁剪区域基于图像 A 定义，然后应用到图像 B，会检查 B 的边界。

    Args:
        img_a_full (numpy.ndarray): 完整背景图 A 数据 (BGR)。
        img_b_full (numpy.ndarray): 完整内容图 B 数据 (BGR)。
        crop_coordinates (tuple): (y_start, y_end, x_start, x_end) 或 None。
                                  -1 表示直到边缘。

    Returns:
        tuple: (img_a_crop, img_b_crop) 裁剪后的图像。
               如果 crop_coordinates 为 None，返回原始图像。
               如果裁剪后尺寸不匹配，会打印警告。
    """
    if crop_coordinates is None:
        print("未提供裁剪坐标，使用完整图像。")
        return img_a_full, img_b_full

    y_start, y_end, x_start, x_end = crop_coordinates
    h_a, w_a = img_a_full.shape[:2]
    h_b, w_b = img_b_full.shape[:2]

    # 规范化 ROI 坐标 (基于 A)
    y_start = max(0, y_start)
    x_start = max(0, x_start)
    y_end = h_a if y_end == -1 else min(h_a, y_end)
    x_end = w_a if x_end == -1 else min(w_a, x_end)
    y_start = min(y_start, y_end)  # 保证 start <= end
    x_start = min(x_start, x_end)

    img_a_crop = img_a_full[y_start:y_end, x_start:x_end]
    print(
        f"图像 A 裁剪至: y[{y_start}:{y_end}], x[{x_start}:{x_end}] -> shape {img_a_crop.shape}"
    )

    # 应用相同绝对坐标到 B (检查 B 边界)
    y_start_b = min(h_b, y_start)
    x_start_b = min(w_b, x_start)
    y_end_b = min(h_b, y_end)
    x_end_b = min(w_b, x_end)
    y_start_b = min(y_start_b, y_end_b)
    x_start_b = min(x_start_b, x_end_b)

    img_b_crop = img_b_full[y_start_b:y_end_b, x_start_b:x_end_b]
    print(
        f"图像 B 裁剪至: y[{y_start_b}:{y_end_b}], x[{x_start_b}:{x_end_b}] -> shape {img_b_crop.shape}"
    )

    # 检查裁剪后尺寸是否一致 (对 absdiff 很重要)
    if img_a_crop.shape[:2] != img_b_crop.shape[:2]:
        print(
            f"警告：裁剪后图像 A ({img_a_crop.shape[:2]}) 和 B ({img_b_crop.shape[:2]}) 尺寸不匹配！"
        )
        print("差异计算可能失败或产生意外结果。")

    return img_a_crop, img_b_crop


def find_difference_and_extract_b_minus_a(
    img_a,
    img_b,
    threshold_value=10,
    output_mask_path=None,
    output_result_path=None,
):
    """
    计算两图差异 (B-A)，生成掩码，提取 B 中不同部分。
    假定 img_a 和 img_b 尺寸相同。

    Args:
        img_a (numpy.ndarray): 背景图 A 数据 (BGR)。
        img_b (numpy.ndarray): 内容图 B 数据 (BGR)。
        threshold_value (int): 像素差异阈值。
        output_mask_path (str, optional): 差异掩码保存路径。
        output_result_path (str, optional): B-A 结果保存路径。

    Returns:
        tuple: (diff_abs, diff_mask, result_b_minus_a) 或 (None, None, None)。
    """
    try:
        # 计算绝对差值
        diff_abs = cv2.absdiff(img_b, img_a)

        # 生成差异掩码 (二值化)
        gray_diff = cv2.cvtColor(diff_abs, cv2.COLOR_BGR2GRAY)
        _, diff_mask = cv2.threshold(gray_diff, threshold_value, 255, cv2.THRESH_BINARY)

        # 可选：形态学去噪 (默认关闭)
        apply_morphology = False
        if apply_morphology:
            kernel = np.ones((3, 3), np.uint8)
            diff_mask = cv2.morphologyEx(diff_mask, cv2.MORPH_OPEN, kernel)
            diff_mask = cv2.morphologyEx(diff_mask, cv2.MORPH_CLOSE, kernel)

        # 生成 B-A 结果
        result_b_minus_a = cv2.bitwise_and(img_b, img_b, mask=diff_mask)

        # 保存中间结果
        if output_mask_path:
            try:
                os.makedirs(os.path.dirname(output_mask_path), exist_ok=True)
                cv2.imwrite(output_mask_path, diff_mask)
            except Exception as e:
                print(f"保存掩码失败: {output_mask_path}, {e}")
        if output_result_path:
            try:
                os.makedirs(os.path.dirname(output_result_path), exist_ok=True)
                cv2.imwrite(output_result_path, result_b_minus_a)
            except Exception as e:
                print(f"保存 B-A 结果失败: {output_result_path}, {e}")

        return diff_abs, diff_mask, result_b_minus_a

    except Exception as e:
        print(f"计算差异时出错: {e}")
        traceback.print_exc()
        return None, None, None


def detect_and_draw_circles(
    mask_image,
    output_image_base,
    dp=1,
    minDist=20,
    param1=50,
    param2=30,
    minRadius=5,
    maxRadius=0,
    output_circles_path=None,
):
    """
    在掩码上检测圆，并在基础图像上绘制。

    Args:
        mask_image (numpy.ndarray): 单通道灰度掩码图。
        output_image_base (numpy.ndarray): 绘制圆的目标 BGR 图 (会创建副本)。
        dp, minDist, ...: HoughCircles 参数。
        output_circles_path (str, optional): 保存绘制结果的路径。

    Returns:
        tuple: (output_image_with_circles, num_circles, circles_data) 或 (None, 0, None)。
    """
    if mask_image is None or output_image_base is None:
        print("错误：圆形检测输入图像无效。")
        return None, 0, None
    if len(mask_image.shape) != 2:
        print(f"错误：圆形检测需要单通道图，输入 shape {mask_image.shape}")
        return None, 0, None

    output_image_with_circles = output_image_base.copy()

    # 预处理 (高斯模糊)
    mask_blurred = cv2.GaussianBlur(mask_image, (9, 9), 2, 2)

    # 霍夫圆检测
    circles = cv2.HoughCircles(
        mask_blurred,
        cv2.HOUGH_GRADIENT,
        dp,
        minDist,
        param1=param1,
        param2=param2,
        minRadius=minRadius,
        maxRadius=maxRadius,
    )

    num_circles = 0
    circles_data = None
    if circles is not None:
        circles = np.uint16(np.around(circles))
        circles_data = circles
        num_circles = circles.shape[1]
        print(f"检测到 {num_circles} 个圆形。")
        for i in circles[0, :]:
            center = (i[0], i[1])
            radius = i[2]
            # 绘制圆
            cv2.circle(output_image_with_circles, center, radius, (0, 255, 0), 2)
            cv2.circle(output_image_with_circles, center, 2, (0, 0, 255), 3)
    else:
        print("未检测到圆形。")

    # 保存结果
    if output_circles_path:
        try:
            os.makedirs(os.path.dirname(output_circles_path), exist_ok=True)
            cv2.imwrite(output_circles_path, output_image_with_circles)
        except Exception as e:
            print(f"保存带圆图像失败: {output_circles_path}, {e}")

    return output_image_with_circles, num_circles, circles_data


def extract_circular_regions(source_image_bgr, circles_data, output_path=None):
    """
    从源图像中提取检测到的圆形区域。

    Args:
        source_image_bgr (numpy.ndarray): 源 BGR 图。
        circles_data (numpy.ndarray): HoughCircles 返回的数据 [[[x,y,r],...]]。
        output_path (str, optional): 提取结果保存路径。

    Returns:
        numpy.ndarray: 包含提取圆形的黑色背景图像，或 None。
    """
    if source_image_bgr is None:
        print("错误：提取区域源图像无效。")
        return None
    if circles_data is None:
        print("警告：无圆形数据，返回黑色图像。")
        return np.zeros_like(source_image_bgr)

    output_image = np.zeros_like(source_image_bgr)
    height, width = source_image_bgr.shape[:2]

    try:
        circles = np.uint16(np.around(circles_data[0, :]))
        print(f"开始提取 {len(circles)} 个圆形区域...")
        for x, y, r in circles:
            if x >= width or y >= height or x < 0 or y < 0 or r <= 0:
                continue  # 边界检查
            temp_mask = np.zeros((height, width), dtype=np.uint8)
            cv2.circle(temp_mask, (x, y), r, (255), thickness=-1)
            extracted_region = cv2.bitwise_and(
                source_image_bgr, source_image_bgr, mask=temp_mask
            )
            output_image = cv2.bitwise_or(output_image, extracted_region)
        print("圆形区域提取完成。")
    except Exception as e:
        print(f"提取圆形区域时出错: {e}")
        # 仍然返回部分结果

    # 保存结果
    if output_path:
        try:
            os.makedirs(os.path.dirname(output_path), exist_ok=True)
            cv2.imwrite(output_path, output_image)
        except Exception as e:
            print(f"保存提取结果失败: {output_path}, {e}")

    return output_image


# --- 主执行逻辑 ---
if __name__ == "__main__":
    # --- 配置 ---
    background_image_file = "depot_test/bg_depot.png"
    test_dir = r"depot_test\Test"
    base_output_dir = "depot_test/output"
    # 注意: crop_coordinates 定义了基于背景图A的感兴趣区域
    crop_coordinates = (150, 1000, 0, -1)  # (y_start, y_end, x_start, x_end) 或 None
    difference_threshold = 20
    hough_dp = 1
    hough_minDist = 100
    hough_param1 = 50
    hough_param2 = 25
    hough_minRadius = 90
    hough_maxRadius = 100
    show_combined_plot = False

    # --- 加载背景图 ---
    if not os.path.exists(background_image_file):
        print(f"错误：背景图片文件不存在: {background_image_file}")
        exit()
    img_a_full = cv2.imread(background_image_file)
    if img_a_full is None:
        print(f"错误：无法加载背景图片: {background_image_file}")
        exit()
    print(f"背景图 {background_image_file} 加载成功: {img_a_full.shape}")

    # --- 查找并处理内容图 ---
    content_image_files = glob.glob(os.path.join(test_dir, "*.png"))

    print(f"开始处理 '{test_dir}' 中的 {len(content_image_files)} 个 PNG 文件...")

    for content_image_file in content_image_files:
        print(f"\n=== 处理文件: {content_image_file} ===")

        img_b_full = cv2.imread(content_image_file)
        if img_b_full is None:
            print(f"错误：无法加载内容图片: {content_image_file}，跳过。")
            continue
        print(f"内容图 {content_image_file} 加载成功: {img_b_full.shape}")

        # --- 步骤 0: 图像裁剪 ---
        print("--- 开始裁剪 ---")
        img_a_crop, img_b_crop = crop_images(img_a_full, img_b_full, crop_coordinates)

        # --- 定义输出子目录和文件名 ---
        base_name_a = os.path.splitext(os.path.basename(background_image_file))[0]
        base_name_b = os.path.splitext(os.path.basename(content_image_file))[0]
        param_str = f"thr{difference_threshold}_dp{hough_dp}_dist{hough_minDist}_p1{hough_param1}_p2{hough_param2}_r{hough_minRadius}-{hough_maxRadius}"

        mask_output_dir = os.path.join(base_output_dir, "Masks")
        result_output_dir = os.path.join(base_output_dir, "Difference_Results")
        circles_output_dir = os.path.join(base_output_dir, "Detected_Circles")
        extracted_output_dir = os.path.join(base_output_dir, "Extracted_Regions")

        filename_mask = f"diff_mask_{base_name_b}_vs_{base_name_a}_{param_str}.png"
        filename_result = f"result_BminusA_{base_name_b}_{param_str}.png"
        filename_circles = f"circles_on_result_{base_name_b}_{param_str}.png"
        filename_extracted = f"extracted_circles_fromB_{base_name_b}_{param_str}.png"

        output_mask_file = os.path.join(mask_output_dir, filename_mask)
        output_result_file = os.path.join(result_output_dir, filename_result)
        output_circles_file = os.path.join(circles_output_dir, filename_circles)
        output_extracted_circles_file = os.path.join(
            extracted_output_dir, filename_extracted
        )

        # --- 执行处理流程 ---
        try:
            # 步骤 1: 计算差异 (B-A)
            print("--- 开始差异计算 ---")
            abs_diff, diff_mask, result_b_minus_a_image = (
                find_difference_and_extract_b_minus_a(
                    img_a_crop,
                    img_b_crop,
                    difference_threshold,
                    output_mask_path=output_mask_file,
                    output_result_path=output_result_file,
                )
            )

            image_with_circles_drawn, num_circles_detected, circles_found_data = (
                None,
                0,
                None,
            )
            extracted_circles_image = None

            # 步骤 2: 圆形检测
            if diff_mask is not None and result_b_minus_a_image is not None:
                print("\n--- 开始圆形检测 ---")
                image_with_circles_drawn, num_circles_detected, circles_found_data = (
                    detect_and_draw_circles(
                        mask_image=diff_mask,
                        output_image_base=result_b_minus_a_image,
                        dp=hough_dp,
                        minDist=hough_minDist,
                        param1=hough_param1,
                        param2=hough_param2,
                        minRadius=hough_minRadius,
                        maxRadius=hough_maxRadius,
                        output_circles_path=output_circles_file,
                    )
                )
            else:
                print("差异计算失败或未生成 B-A 结果，跳过圆形检测。")

            # 步骤 3: 提取圆形区域
            if circles_found_data is not None and img_b_crop is not None:
                print("\n--- 开始提取圆形区域 ---")
                extracted_circles_image = extract_circular_regions(
                    source_image_bgr=img_b_crop,
                    circles_data=circles_found_data,
                    output_path=output_extracted_circles_file,
                )
            elif diff_mask is not None and result_b_minus_a_image is not None:
                print("未检测到圆形，跳过区域提取。")
                if img_b_crop is not None:
                    extracted_circles_image = np.zeros_like(img_b_crop)

            # 步骤 4: 显示结果 (可选)
            if show_combined_plot:
                # Create lists *without* img_a_crop and its title
                images_to_plot = [
                    img_b_crop,
                    abs_diff,
                    diff_mask,
                    result_b_minus_a_image,
                    image_with_circles_drawn,
                    extracted_circles_image,
                ]
                titles = [
                    "B (裁剪)",
                    "|B - A|",
                    f"掩码(thr={difference_threshold})",
                    "结果 (B - A)",
                    f"检测圆 ({num_circles_detected})",
                    "提取区域 (from B)",
                ]
                valid_plots = [
                    (img, title)
                    for img, title in zip(images_to_plot, titles)
                    if img is not None
                ]

                if valid_plots:
                    num_plots = len(valid_plots)
                    rows = 1 if num_plots <= 3 else 2
                    cols = (
                        num_plots + rows - 1
                    ) // rows  # Calculate cols based on rows

                    plt.figure(figsize=(cols * 5, rows * 4))  # Adjust figsize if needed
                    for i, (img, title) in enumerate(valid_plots):
                        plt.subplot(rows, cols, i + 1)
                        if len(img.shape) == 2:
                            plt.imshow(img, cmap="gray")
                        else:
                            plt.imshow(cv2.cvtColor(img, cv2.COLOR_BGR2RGB))
                        plt.title(title)
                        plt.axis("off")
                    plt.suptitle(
                        f"处理结果: {os.path.basename(content_image_file)}", fontsize=14
                    )
                    plt.tight_layout(rect=[0, 0.03, 1, 0.95])
                    plt.show()
                else:
                    print("无有效图像可供显示。")
            else:
                print("已禁用 Matplotlib 绘图显示。")

        except Exception as e:
            print(f"处理文件 {content_image_file} 时发生严重错误: {e}")
            traceback.print_exc()
            print(f"跳过文件 {content_image_file} 的后续处理。")
            continue

    print("\n所有文件处理完毕。")
