from typing import Literal

import cv2

from mower.utils import config
from mower.utils.image import cropimg, loadres, template
from mower.utils.log import logger
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import sa, vs

from .utils import generate_name


class ActivityNavigation(BaseSolver):
    solver_name = "活动关卡导航"
    prefix = "RS"

    def generate_names(self):
        names = {}
        targets = (
            [
                "-TR-1",
                "-TR-2",
                "-TR-3",
                "-1",
                "-2",
                "-3",
                "-4",
                "-5",
                "-6",
                "-7",
                "-8",
                "-9",
                "-10",
            ]
            if self.difficulty != "ex"
            else [
                "-EX-1",
                "-EX-2",
                "-EX-3",
                "-EX-4",
                "-EX-5",
                "-EX-6",
                "-EX-7",
                "-EX-8",
                "-EX-9",
                "-EX-10",
            ]
        )
        for name in targets:
            name = self.prefix + name
            font_size = 37 if len(name) > 5 else 40
            style = "dark" if "TR-" in name else "light"
            names[name] = generate_name(name, font_size, pad_right=10, style=style)
        self.names = names

    def get_stage_pos(self, scope=((100, 150), (1820, 790))):
        img = cropimg(config.recog.img, scope)
        score, scope = template(self.whole_scene, img)
        logger.debug(f"{score=} {scope=} {self.stage_scope=}")
        if score < 0.5:
            return None
        res_scope = vs(self.stage_scope[0], scope[0]), vs(self.stage_scope[1], scope[0])
        res_scope = sa(res_scope, (100, 150))
        return res_scope

    def run(
        self,
        name: str,
        mode: Literal["auto", "copy"] = "auto",
        switch_to_ex: bool = False,
    ) -> None:
        logger.info("导航至活动关卡")
        self.switch_to_ex = switch_to_ex  # 是否切换到突袭模式
        self.difficulty = "ex" if "-EX-" in name else "normal"
        self.name = name
        self.mode = mode
        self.generate_names()
        self.whole_scene = loadres(f"navigation/whole_scene/activity_{self.difficulty}")
        self.stage_scope = template(
            cv2.cvtColor(self.whole_scene, cv2.COLOR_RGB2GRAY), self.names[self.name]
        )[1]
        super().run()
        return True

    def transition(self) -> bool:
        if (scene := self.scene()) == Scene.TERMINAL_MAIN:
            self.terminal_entry("navigation/activity/terminal.jpg")
        elif scene == Scene.ACTIVITY_MAIN:
            self.tap(f"navigation/activity/entry_{self.difficulty}")
        elif scene in [
            Scene.ACTIVITY_NORMAL_CHOOSE_LEVEL,
            Scene.ACTIVITY_EX_CHOOSE_LEVEL,
        ]:
            if (
                self.mode == "copy"
                and (result := config.recog.match("story_stage"))
                and result[0] > 0.8
            ):
                self.ctap(result[1], 3)
                return
            stage_pos = self.get_stage_pos()
            if stage_pos[1][0] < 0:
                self.swipe_ext(
                    [(400, 540), (1400, 540), (1400, 800)], [200, 300], 0, 0.1
                )
            elif stage_pos[0][0] > 1920:
                self.swipe_ext(
                    [(1400, 540), (400, 540), (400, 800)], [200, 300], 0, 0.1
                )
            else:
                self.ctap(stage_pos, 3)

        elif scene == Scene.OPERATOR_BEFORE:
            if self.find("ope_ex_mode") and self.switch_to_ex:
                self.ctap("ope_ex_mode", 3)
                return

            if self.mode == "auto":
                return True
            elif self.mode == "copy":
                if not self.switch_to_ex:
                    if config.recog.gray[65][1333] < 200:
                        self.sleep()
                        return
                    # 取消代理作战
                    if config.recog.gray[907][1600] > 127 and not self.find(
                        "ope_agency_lock"
                    ):
                        self.ctap((1776, 908), 3)
                        return
                    self.ctap("ope_start", 3)
                else:
                    self.ctap("ope_ex_start", 3)
        elif scene == Scene.OPERATOR_SELECT and self.mode == "copy":
            return True
        elif scene in [Scene.OPERATOR_RECOVER_ORIGINITE, Scene.OPERATOR_RECOVER_POTION]:
            return True
        elif scene == Scene.STORY_STAGE:
            self.tap("start_story")

        else:
            self.scene_graph_step(Scene.TERMINAL_MAIN)
