from difflib import SequenceMatcher

import cv2

from mower.solvers.navigation.score import Score
from mower.static import secret_front
from mower.utils import config
from mower.utils import typealias as tp
from mower.utils.email import notify
from mower.utils.image import cropimg
from mower.utils.log import logger
from mower.utils.rapidocr import ocr_rec
from mower.utils.scene import Scene
from mower.utils.solver import BaseSolver
from mower.utils.vector import sa, va

CardInfo = tuple[int, int, int, float]


def exp(card: CardInfo) -> list[float]:
    """计算卡片的期望

    Args:
        card (CardInfo): 卡片的值

    Returns:
        list[float]: 三项属性的期望
    """
    return [i * card[3] for i in card[:3]]


class SecretFront(BaseSolver):
    solver_name = "隐秘战线"

    target = {
        "1A": [20, 20, 20],
        "2A": [60, 55, 45],
        "2B": [45, 55, 65],
        "3A": [200, 155, 155],
        "3B": [150, 220, 250],
        "3C": [155, 255, 240],
        "结局A": [510, 350, 350],
        "结局B": [440, 440, 350],
        "结局C": [350, 550, 350],
        "结局D": [365, 475, 495],
        "结局E": [370, 385, 620],
    }
    routes = {
        "结局A": ["1A", "2A", "3A", "结局A"],
        "结局B": ["1A", "2A", "3A", "结局B"],
        "结局C": ["1A", "2A", "3B", "结局C"],
        "结局D": ["1A", "2B", "3C", "结局D"],
        "结局E": ["1A", "2B", "3C", "结局E"],
    }
    teams = {
        "结局A": "management",
        "结局B": "management",
        "结局C": "intelligence",
        "结局D": "medicine",
        "结局E": "medicine",
    }

    def __init__(self):
        self.properties: list[int] | None = None
        "三项属性值"
        self.event: bool = False
        "应对危机事件"
        self.route_names: list[str] = []
        "匹配路线"
        self.series: int | None = None
        "支援作战平台、游侠、诡影迷踪"
        self.actions: dict[int, dict[int, float]] = {}
        "记录页面中卡片的期望"
        self.send_email: bool = False
        "发送邮件提醒"

        super().__init__()

    @property
    def route(self):
        "路线"
        return self.routes[config.conf.secret_front.target]

    @property
    def team(self):
        "小队"
        return self.teams[config.conf.secret_front.target]

    def run(self):
        self.route_names = []

        self.event = False
        self.series = None

        self.send_email = False

        self.reset_actions()

        return super().run()

    def reset_actions(self):
        "清空缓存的卡片数据"
        self.actions = {0: {}, 1: {}, 2: {}}

    def number(self, scope: tp.Scope, height: int = 25):
        return config.recog.num.number_int("secret_front", scope, height)

    def card_pos(self, total: int, idx: int) -> tp.Coordinate:
        """总共有total张卡片时下标为idx（从0开始）卡片的左上角坐标

        Args:
            total (int): 卡片总数
            idx (int): 下标（从0开始）

        Returns:
            tp.Coordinate: 卡片左上角坐标
        """
        return {
            1: [(830, 466)],
            2: [(565, 466), (1095, 466)],
            3: [(301, 466), (830, 466), (1360, 466)],
        }[total][idx]

    def event_card(self, total: int, idx: int) -> tuple[str, bool]:
        """应对危机事件卡片识别

        Args:
            total (int): 卡片总数
            idx (int): 卡片下标（从0开始）

        Returns:
            tuple[str, bool]: 危机事件选项名称及是否可选
        """
        pos = self.card_pos(total, idx)

        scope = sa(((10, 380), (140, 430)), pos)
        img = cropimg(config.recog.gray, scope)
        img = cv2.copyMakeBorder(img, 10, 10, 10, 10, cv2.BORDER_CONSTANT, None, (0,))
        score = []
        for i in self.target:
            result = cv2.matchTemplate(img, secret_front()[i], cv2.TM_SQDIFF_NORMED)
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
            score.append(min_val)
        name = list(self.target)[score.index(min(score))]

        x, y = va(pos, (350, 460))
        hue = config.recog.hsv[y][x][0]

        logger.debug(f"{name=} {hue=}")
        return name, hue > 18

    def card(self, total: int, idx: int) -> CardInfo:
        """卡片识别

        Args:
            total (int): 总数
            idx (int): 下标（从0开始）

        Returns:
            CardInfo: 卡片信息
        """
        pos = self.card_pos(total, idx)

        materiel = sa(((84, 70), (180, 102)), pos)
        intelligence = sa(((249, 70), (343, 102)), pos)
        medicine = sa(((414, 70), (470, 102)), pos)
        percentage = sa([[10, 440], [130, 473]], pos)

        materiel = self.number(materiel)
        intelligence = self.number(intelligence)
        medicine = self.number(medicine)

        min_val, min_loc = self.template_match(
            "sf/percentage", percentage, cv2.TM_SQDIFF_NORMED
        )
        percentage[1][0] = min_loc[0][0]
        percentage = self.number(percentage, 28) / 100

        logger.debug(f"{materiel=} {intelligence=} {medicine=} {percentage=}")

        return materiel, intelligence, medicine, percentage

    def page_number(self) -> int | None:
        """识别当前页数

        Returns:
            int | None: 页数（从0开始）
        """
        if not self.route_names:
            self.tap((125, 840))
            return None
        img = cropimg(config.recog.gray, ((915, 230), (1320, 290)))
        name = ocr_rec(img)
        max_ratio = 0
        page_number = 0
        for i, n in enumerate(self.route_names):
            if (ratio := SequenceMatcher(None, name, n).ratio()) > max_ratio:
                page_number = i
                max_ratio = ratio
        logger.debug(f"{page_number=}")
        return page_number

    def card_total(self) -> int:
        """识别当前有几张卡片

        Returns:
            int: 卡片数量
        """
        p3 = self.card_pos(3, 0)
        p2 = self.card_pos(2, 0)
        up_scope = ((0, 0), (473, 120))
        down_scope = ((0, 432), (150, 474))
        s3u = sa(up_scope, p3)
        s3d = sa(down_scope, p3)
        s2u = sa(up_scope, p2)
        s2d = sa(down_scope, p2)
        if self.find("sf/card", scope=s3u):
            total = 3
        elif self.find("sf/available", scope=s3d):
            total = 3
        elif self.find("sf/card", scope=s2u):
            total = 2
        elif self.find("sf/available", scope=s2d):
            total = 2
        else:
            total = 1
        logger.debug(f"{total=}")
        return total

    def max_card(self) -> tuple[int, int]:
        """计算最优选项

        Returns:
            tuple[int, int]: 返回最优页码及卡片下标
        """
        max_page = -1
        max_card = -1
        max_score = -1

        logger.debug(f"{self.properties=}")

        for page, page_action in self.actions.items():
            for card, action in page_action.items():
                for stage in self.route:
                    target = self.target[stage]
                    if all(p < t for p, t in zip(self.properties, target)):
                        break
                distance = [max(t - p, 0) for p, t in zip(self.properties, target)]
                if sum(distance) == 0:
                    return 0, 0
                total_score = 0
                for i in range(3):
                    score = action[i] * distance[i] / sum(distance)
                    total_score += score
                logger.debug(f"{page=} {card=} {stage=} {total_score=}")
                if total_score > max_score:
                    max_score = total_score
                    max_page = page
                    max_card = card
        logger.debug(f"{max_page=} {max_card=}")
        return max_page, max_card

    def choose_card(self, total: int, idx: int):
        """点击卡片

        Args:
            total (int): 卡片总数
            idx (int): 卡片下标（从0开始）
        """
        start = {3: 545, 2: 805, 1: 1075}[total]
        pos = (start + idx * 530, 900)
        self.tap(pos)

    def transition(self):
        if (scene := self.scene()) == Scene.TERMINAL_SCORE:
            Score().run("14-1")
        elif scene == Scene.OPERATOR_CHOOSE_LEVEL:
            if pos := self.find("sf/entrance"):
                self.tap(pos)
                return
            if self.animation():
                return
            self.scene_graph_step(Scene.TERMINAL_SCORE)
        elif scene == Scene.SF_SELECT_TEAM:
            if self.animation():
                return
            if pos := self.find(f"sf/team_{self.team}"):
                self.tap(pos)
                return
            self.tap("sf/select_team_ok")
        elif scene == Scene.SF_CONTINUE:
            self.tap("sf/continue")
        elif scene == Scene.SF_SELECT:
            if self.animation(((0, 0), (1920, 1030))):
                return
            self.send_email = True
            self.event = False

            if self.properties is None:
                self.properties = [
                    self.number(((105, 490), (180, 525)), 23),
                    self.number(((105, 580), (180, 615)), 23),
                    self.number(((105, 675), (180, 705)), 23),
                ]
                logger.debug(f"{self.properties=}")

            if not self.route_names:
                for i in range(3):
                    pos = self.card_pos(3, i)
                    scope = sa(((120, 145), (460, 205)), pos)
                    img = cropimg(config.recog.gray, scope)
                    self.route_names.append(ocr_rec(img))

            self.series = None
            if (
                (pos := self.find("sf/support_battle_platform"))
                or (pos := self.find("sf/ranger"))
                or (
                    self.route[-1] == "结局E"
                    and (pos := self.find("sf/lost_in_the_trick"))
                )
            ):
                if (pos_x := pos[0][0]) < 800:
                    self.series = 0
                elif pos_x < 1330:
                    self.series = 1
                else:
                    self.series = 2
                logger.debug(f"{self.series=}")

                self.tap((545 + 530 * self.series, 640))
                return

            self.tap((545, 640))
        elif scene == Scene.SF_ACTIONS:
            if self.event and (pos := self.find("sf/event_confirm")):
                self.tap(pos)
                return
            elif pos := self.find("sf/card_confirm"):
                self.tap(pos)
                return

            if self.animation(((0, 0), (1920, 1030))):
                return

            total = self.card_total()

            if self.event:
                name_list = [self.event_card(total, i) for i in range(total)]
                for idx, (name, available) in enumerate(name_list):
                    if name not in self.route:
                        continue
                    if available:
                        self.choose_card(total, idx)
                        return
                    break
                self.tap("sf/exit_button")
                return

            if (page_number := self.page_number()) is None:
                return

            target = self.target[self.route[-1]]
            distance = [max(t - p, 0) for p, t in zip(self.properties, target)]
            if sum(distance) == 0:
                self.choose_card(total, 0)
                return

            if self.series is not None:
                card_data = [self.card(total, idx) for idx in range(total)]
                for idx in range(total):
                    self.actions[page_number][idx] = exp(card_data[idx])
                if all(card[3] < 0.8 for card in card_data):
                    logger.debug("成功概率太低")
                    self.series = None
                    self.solver_update_before_transition = False
                    return

            elif not all(self.actions.values()):
                # 先看一遍所有行动，找到还没看过的页。
                target_number = next(i for i in range(3) if not self.actions[i])
                if target_number == page_number:
                    # 如果要读的是当前页，读取并计算分数
                    for idx in range(total):
                        self.actions[page_number][idx] = exp(self.card(total, idx))
                    logger.debug(f"{self.actions=}")
                    self.solver_update_before_transition = False
                elif (page_number + 1) % 3 == target_number:
                    self.tap((1785, 225))  # 下一页
                else:
                    self.tap((350, 225))  # 上一页
                return

            max_page, max_card = self.max_card()

            if max_page == page_number:
                self.choose_card(len(self.actions[max_page]), max_card)
            elif (page_number + 1) % 3 == max_page:
                self.tap((1785, 225))  # 下一页
            else:
                self.tap((350, 225))  # 上一页
        elif scene == Scene.SF_RESULT:
            self.route_names = []
            self.properties = None
            self.reset_actions()
            self.tap((1640, 945))
        elif scene == Scene.SF_EVENT:
            self.event = True
            self.tap("sf/continue_event")
        elif scene in [Scene.SF_TEAM_PASS, Scene.SF_CLICK_ANYWHERE, Scene.SF_END]:
            self.tap((960, 980))

            if scene == Scene.SF_END and self.send_email:
                self.send_email = False
                notify(f"隐秘战线通关{config.conf.secret_front.target}", level="INFO")

                if config.conf.secret_front.auto_next:
                    if config.conf.secret_front.target != "结局E":
                        targets = list(self.routes.keys())
                        index = targets.index(config.conf.secret_front.target)
                        config.conf.secret_front.target = targets[index + 1]
                        config.save_conf()
                        return
                if config.conf.secret_front.stop_after_finish:
                    config.conf.task["mower.solvers.long_task.LongTask"] = False
                    config.save_conf()
                    return True
        elif scene == Scene.SF_EXIT:
            self.tap("sf/restart")
        else:
            self.scene_graph_step(Scene.TERMINAL_SCORE)
