from mower.data import material_calculation_rules


class MaterialNode:
    def __init__(self, name):
        self.name = name
        self.children = []

    def add_child(self, child_node, count):
        self.children.append((child_node, count))


class MaterialTree:
    def __init__(self, rules):
        self.nodes = {}
        self._build_tree(rules)

    def _build_tree(self, rules):
        """根据规则构建材料树"""
        for material, children in rules.items():
            # 创建或获取当前材料节点
            if material not in self.nodes:
                self.nodes[material] = MaterialNode(material)

            current_node = self.nodes[material]

            # 添加子节点
            for child_name, count in children:
                if child_name not in self.nodes:
                    self.nodes[child_name] = MaterialNode(child_name)

                child_node = self.nodes[child_name]
                current_node.add_child(child_node, count)


class Planner:
    def __init__(self, material_tree):
        self.material_tree = material_tree

    def generate_plan(self, requirements, have=None):
        """
        生成合成计划
        :param requirements: 需求字典 {材料名: 数量}
        :param have: 库存字典 {材料名: 数量}
        :return: (基础材料需求字典, 合成步骤列表)
        """
        # 使用副本避免修改原始库存数据
        have = have.copy() if have else {}
        total_base = {}
        all_steps = []

        for target, demand in requirements.items():
            if target not in self.material_tree.nodes:
                raise ValueError(f"未定义的合成材料: {target}")

            # 处理当前材料需求
            node = self.material_tree.nodes[target]
            base_needs, steps = self._calculate_requirements(node, demand, have)

            # 合并基础材料需求
            for material, count in base_needs.items():
                total_base[material] = total_base.get(material, 0) + count

            # 收集合成步骤
            all_steps.extend(steps)

        return total_base, all_steps

    def _calculate_requirements(self, node, demand, have):
        """递归计算材料需求"""
        # 先用库存满足需求
        available = have.get(node.name, 0)
        if available >= demand:
            have[node.name] -= demand
            return {}, []

        # 计算需要合成的数量
        remaining = demand - available
        have[node.name] = 0  # 清空库存

        # 如果是基础材料（没有配方）
        if not node.children:
            return {node.name: remaining}, []

        # 计算子材料需求
        total_base = {}
        child_steps = []

        for child_node, unit_need in node.children:
            # 总需求 = 当前需要合成的数量 × 每个需要的数量
            total_need = remaining * unit_need
            child_base, steps = self._calculate_requirements(
                child_node, total_need, have
            )

            # 合并基础材料需求
            for material, count in child_base.items():
                total_base[material] = total_base.get(material, 0) + count

            # 收集子步骤
            child_steps.extend(steps)

        # 添加当前合成步骤
        current_step = {
            "target": node.name,
            "count": remaining,
            "materials": {child.name: cnt for child, cnt in node.children},
        }
        child_steps.append(current_step)

        return total_base, child_steps


# 使用示例
if __name__ == "__main__":
    # 加载合成规则
    rules = material_calculation_rules

    # 初始化材料和规划器
    material_tree = MaterialTree(rules)
    planner = Planner(material_tree)

    # 用户输入
    requirements = {"D32钢": 5}
    inventory = {"三水锰矿": 1, "轻锰矿": 1}

    try:
        # 生成计划
        base_materials, steps = planner.generate_plan(requirements, inventory)

        # 获取所有基础材料列表
        base_items = [
            name for name, node in material_tree.nodes.items() if not node.children
        ]

        # 生成完整基础材料需求（包含需求为0的）
        full_base = {item: base_materials.get(item, 0) for item in base_items}

        # 显示结果
        print("========== 合成规划 ==========")
        print(f"目标需求: {requirements}")
        print(f"初始库存: {inventory}\n")

        print("----- 基础材料需求 -----")
        for item, need in full_base.items():
            if need > 0:
                original = inventory.get(item, 0)
                print(f"· {item}: 需补充 {need} 个 (库存 {original} 个)")

        print("\n----- 合成步骤 -----")
        for step in reversed(steps):  # 反转步骤顺序使根步骤在前
            materials = [f"{k}×{v}" for k, v in step["materials"].items()]
            print(
                f"合成 [{step['target']}] ×{step['count']} → 需要: {', '.join(materials)}"
            )

    except ValueError as e:
        print(f"错误: {e}")
