"""
Copyright (c) 2024 Elaina <2901432375@qq.com>
Copyright (c) 2024 zhbaor <zhbaor@zhaozuohong.vip>

This file is part of mower-ng (https://git.zhaozuohong.vip/mower-ng/mower-ng).

Mower-ng is free software: you may copy, redistribute and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation, version 3 or later.

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
"""

import gc
from bisect import insort
from datetime import datetime, time, timedelta

from pydantic import BaseModel

from mower.static import clear_data
from mower.utils import config
from mower.utils.csleep import MowerExit, csleep
from mower.utils.datetime import format_time
from mower.utils.email import send_message, task_template
from mower.utils.import_helper import check_import, import_class
from mower.utils.log import logger
from mower.utils.translate import translate_room

now = datetime.now()


class BasicTask(BaseModel):
    "内部使用"

    solver: str
    "调用solver().run()"

    priority: int
    "数字越小，优先级越高"

    schedule_time: datetime | None = None
    "调度器指定的时间"

    @property
    def _solver_name(self):
        if check_import(self.solver):
            solver_class = import_class(self.solver)
            if solver_class.solver_name is None:
                return solver_class.__qualname__
            return solver_class.solver_name
        else:
            return self.solver

    def _get_next_execution(self) -> datetime:
        """任务内部使用，子类可覆盖此方法，计算下一次运行任务的时间

        Returns:
            datetime: 下一次运行的时间
        """
        return now

    def next_execution(self) -> datetime:
        """提供给外部的方法，用于获取下一次运行的时间

        Returns:
            datetime: 下一次运行的时间
        """
        if self.schedule_time:
            return self.schedule_time
        return self._get_next_execution()

    def __lt__(self, other: "BasicTask"):
        self_time = self.next_execution()
        other_time = other.next_execution()

        if self_time <= now and other_time <= now:
            # 两边都小于当前时间或时间相同时按优先级比较
            return self.priority < other.priority
        return self_time < other_time  # 按时间比较


class OneTimeTask(BasicTask):
    "精确的任务，如基建任务"

    time: datetime

    def __str__(self):
        return f"定时任务[{self._solver_name}] 运行时间{self.time}"

    def _get_next_execution(self):
        return self.time


class PeriodicTask(BasicTask):
    "多次执行、不低于一定间隔的任务，如清理智、公招"

    interval: timedelta
    "执行间隔"
    _last_execution: datetime | None = None

    def __str__(self):
        return f"周期任务[{self._solver_name}] 运行周期{self.interval}"

    def _get_next_execution(self):
        if self._last_execution is None:
            return now
        else:
            return self._last_execution + self.interval


class DailyTask(BasicTask):
    "每日任务，如签到"

    time_offset: timedelta
    "每日最早执行的时间"
    _last_execution: datetime | None = None

    def __str__(self):
        return f"每日任务[{self._solver_name}] 每日{self.time_offset}后运行"

    def _get_next_execution(self):
        if self._last_execution is None:
            return now
        else:
            result = (
                datetime.combine(self._last_execution.date(), time()) + self.time_offset
            )
            if self._last_execution > result:
                result += timedelta(days=1)
            return result


class LongTask(BasicTask):
    "大型任务，如肉鸽、保全"

    def __str__(self):
        return f"大型任务[{self._solver_name}]"


Task = OneTimeTask | PeriodicTask | DailyTask | LongTask


class Scheduler:
    def __init__(self):
        self.priority_queue: list[Task] = []  # 小于当前时间的待执行队列，按优先级排序
        self.time_queue: list[Task] = []  # 未启用的或大于当前时间的等待队列，按时间排序

    @property
    def waiting_tasks(self):
        return filter(lambda t: config.conf.task[t.solver], self.time_queue)

    @property
    def next_time(self) -> datetime | None:
        for next_task in self.waiting_tasks:
            if (
                not config.conf.postpone_non_riic
                or next_task.solver == "mower.solvers.base_manager.BaseManager"
            ):
                return next_task.next_execution()
        return None

    def register(self, task: Task):
        logger.info(f"注册任务：{task}")
        insort(self.time_queue, task)

    def update_queue(self):
        tasks, self.priority_queue = self.priority_queue, []
        for task in tasks:
            if config.conf.task[task.solver]:
                self.priority_queue.append(task)
            else:
                insort(self.time_queue, task)
        tasks, self.time_queue = self.time_queue, []
        for task in tasks:
            if config.conf.task[task.solver] and task.next_execution() <= now:
                insort(self.priority_queue, task)
            else:
                self.time_queue.append(task)

    def register_one_time(self, solver: str, time: datetime, priority: int = 10):
        self.register(OneTimeTask(solver=solver, time=time, priority=priority))

    def register_periodic(self, solver: str, interval: timedelta, priority: int = 20):
        self.register(PeriodicTask(solver=solver, interval=interval, priority=priority))

    def register_daily(self, solver: str, offset: timedelta, priority: int = 30):
        self.register(DailyTask(solver=solver, time_offset=offset, priority=priority))

    def register_long(self, solver: str, priority: int = 40):
        self.register(LongTask(solver=solver, priority=priority))

    def list_tasks(self):
        output = [f"共{len(self.priority_queue + self.time_queue)}项任务"]
        for i, task in enumerate(self.priority_queue):
            output.append(
                f"执行队列：({i + 1:2}/{len(self.priority_queue)}) {task.next_execution().strftime('%H:%M:%S')} {task.priority} {task._solver_name}"
            )
        for i, task in enumerate(self.time_queue):
            output.append(
                f"等待队列：({i + 1:2}/{len(self.time_queue)}) {task.next_execution().strftime('%H:%M:%S')} {task.priority} {task._solver_name}"
            )
        logger.info("\n".join(output))

    def schedule(self):
        global now
        while True:
            try:
                if len(self.priority_queue + self.time_queue) == 0:
                    logger.info("任务列表为空，停止运行调度器")
                    config.idle = True
                    return
                now = datetime.now()
                self.update_queue()
                self.list_tasks()
                if self.priority_queue:
                    next_task = self.priority_queue.pop(0)
                    solver = import_class(next_task.solver)()
                    stop_time = None
                    for task in self.waiting_tasks:
                        if task.priority < next_task.priority:
                            stop_time = task.next_execution()
                            break

                    solver.scheduler_stop_time = stop_time
                    logger.info(f"开始运行{next_task}，结束时间为{stop_time}")
                    try:
                        complete = solver.run()
                    except MowerExit:
                        raise
                    except Exception as e:
                        logger.exception(f"{next_task._solver_name}任务运行出错：{e}")
                        complete = True
                    clear_data()
                    gc.collect()
                    if complete:
                        if isinstance(next_task, (PeriodicTask, DailyTask)):
                            next_task._last_execution = datetime.now()
                            next_task.schedule_time = None
                            insort(self.time_queue, next_task)
                        elif isinstance(next_task, LongTask):
                            next_task.schedule_time = stop_time
                            insort(self.time_queue, next_task)
                    else:
                        next_task.schedule_time = stop_time
                        insort(self.time_queue, next_task)
                else:
                    # 如果没有需要立即执行的任务，休息
                    next_time = self.next_time
                    if next_time is None:
                        raise MowerExit
                    sleep_time = next_time - now
                    sleep_time_display = format_time(sleep_time.total_seconds())
                    if sleep_time > timedelta(minutes=3):
                        config.idle = True
                        if config.device:
                            config.device.idle_solver()
                        title = f"休息{sleep_time_display}，下次任务在{next_time.strftime('%H:%M:%S')}"
                        logger.info(title)
                        config.task_count += 1
                        logger.info(f"第{config.task_count}次任务结束")
                        send_message(
                            task_template.render(
                                scheduler_tasks=self.waiting_tasks,
                                riic_tasks=[task.format() for task in config.tasks],
                                translate_room=translate_room,
                            ),
                            title,
                            "INFO",
                        )
                    else:
                        logger.info(f"下次任务时间较近，等待{sleep_time_display}")
                    csleep(sleep_time.total_seconds())
                    config.idle = False
                    if config.recog:
                        config.recog.update()

            except MowerExit:
                logger.info("停止运行调度器")
                return
            except Exception as e:
                logger.exception(f"调度器报错退出：{e}")
                return


scheduler = Scheduler()
